/*
 * Created on 2006/01/21
 *
 * Copyright (c) 2005 Nooto Poject.  All rights reserved.
 */
package camei.select2;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.MouseInputListener;

/**
 * SelectElementHandler̎gpTvB
 * 
 * @see SelectElementHandler
 * @author camei
 * @version $Id: SelectSample.java,v 1.2 2006/01/25 11:56:25 camei Exp $
 */
public class SelectSample extends JFrame implements SelectionManager {

	private List elementList = new LinkedList();
	private List selectedElementList = new LinkedList();
	private List preselectedElementList = new LinkedList();
	private SelectElementHandler handler;

	private JComponent canvas = new JPanel() {
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			g.setColor(Color.GRAY);
			((Graphics2D) g).draw(selectionBounds);
		}
	};
	private int labelNo = 0;
	private Rectangle selectionBounds = new Rectangle();

	/**
	 * RXgN^B
	 */
	public SelectSample() {
		super("test");
		handler = new SelectElementHandler(this);

		for (int i = 0; i < 3; i++) {
			createElement();
		}
		canvas.addMouseListener(handler);
		canvas.addMouseMotionListener(handler);
		canvas.setBackground(Color.WHITE);
		canvas.setOpaque(true);
		getContentPane().add(canvas);

		addWindowListener(new WindowAdapter() {
			public void windowClosed(WindowEvent e) {
				System.exit(0);
			}
		});
		setSize(600, 400);
		setVisible(true);
	}

	private IconElement createElement() {
		IconElement instance = new IconElement(labelNo++);
		elementList.add(instance);
		canvas.add(instance);
		return instance;
	}

	private IconElement copyElement(IconElement element) {
		IconElement instance = new IconElement(element);
		elementList.add(instance);
		canvas.add(instance);
		return instance;
	}

	private int getSelectModifiers(MouseEvent e) {
		if (e.isControlDown()) {
			return SelectElementHandler.ADD;
		} else {
			return SelectElementHandler.SIMPLE;
		}
	}

	/**
	 * vf
	 * 
	 * @author camei
	 * @version $Id: SelectSample.java,v 1.2 2006/01/25 11:56:25 camei Exp $
	 */
	private class IconElement extends JLabel {
		private int number = -1;
		private boolean isEditable = false;
		private JTextField textField = new JTextField();


		public IconElement(int number) {
			super("element" + number);
			this.number = number;
			setBounds(10 + 100 * number, 100, 80, 30);
			setPreferredSize(new Dimension(80, 30));
			MouseInputListener l = handler.getComponentHandler(this);
			addMouseListener(l);
			addMouseMotionListener(l);
			add(textField);
		}

		public IconElement(IconElement element) {
			this(element.number);
			setBounds(element.getBounds());
		}

		public void paintComponent(Graphics g) {
			if (handler.isAddSelectingBounds()) {
				if (preselectedElementList.contains(this)) {
					g.setColor(Color.BLUE);
				} else {
					g.setColor(Color.GRAY);
				}
			} else {
				if (selectedElementList.contains(this)
						|| preselectedElementList.contains(this)) {
					g.setColor(Color.BLUE);
				} else {
					g.setColor(Color.GRAY);
				}
			}
			Dimension d = getSize();
			g.drawRect(0, 0, d.width - 1, d.height - 1);
			super.paintComponent(g);
		}

		public void translate(int dx, int dy) {
			Rectangle bounds = getBounds();
			bounds.x += dx;
			bounds.y += dy;
			setBounds(bounds);
		}

		
		
		public void setEditale(boolean isEditable) {
			if (!this.isEditable && isEditable) {// ҏWԂɂ
				textField.setText(getText());
				textField.setVisible(true);
			} else if (this.isEditable && !isEditable) {// ҏW肷
				setText(textField.getText());
				textField.setVisible(false);
			}

			this.isEditable = isEditable;
		}

	}

	/**
	 * s
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		new SelectSample();
	}

	/***************************************************************************
	 * SelectManager ̎
	 **************************************************************************/

	public void select(Object element) {
		selectedElementList.add(element);
		repaint();
	}

	public void unselect(Object element) {
		selectedElementList.remove(element);
		repaint();
	}

	public void clearSelection() {
		selectedElementList.clear();
		repaint();
	}

	public boolean isSelected(Object element) {
		return selectedElementList.contains(element);
	}

	public Object getElement(MouseEvent e) {
		return null;
	}

	public int getSelectOption(MouseEvent e) {
		if (e.isControlDown()) {
			return SelectElementHandler.ADD;
		} else {
			return SelectElementHandler.SIMPLE;
		}
	}

	public Point getGlobalPoint(MouseEvent e) {
		return e.getPoint();
	}

	public void readySelectBounds() {
		System.out.println("readySelectBounds");

	}

	public void selectBounds(int x1, int y1, int x2, int y2) {
		System.out.println("selectBounds");
		createSelectionBounds(x1, y1, x2, y2);

		preselectedElementList.clear();
		for (Iterator i = elementList.iterator(); i.hasNext();) {
			IconElement element = (IconElement) i.next();
			if (selectionBounds.intersects(element.getBounds())) {
				preselectedElementList.add(element);
			}
		}
		repaint();
	}

	public void decideSelectBoudns() {
		System.out.println("decideSelectBounds");
		selectionBounds.setBounds(0, 0, 0, 0);
		selectedElementList.clear();
		selectedElementList.addAll(preselectedElementList);
		preselectedElementList.clear();
		repaint();
	}

	private void createSelectionBounds(int x1, int y1, int x2, int y2) {
		selectionBounds.x = (x1 < x2) ? x1 : x2;
		selectionBounds.y = (y1 < y2) ? y1 : y2;
		selectionBounds.width = (x1 < x2) ? x2 - x1 : x1 - x2;
		selectionBounds.height = (y1 < y2) ? y2 - y1 : y1 - y2;

	}

	public void readySelectBoundsAdditional() {
		// TODO ꂽ\bhEX^u

	}

	public void selectBoundsAdditional(int x1, int y1, int x2, int y2) {
		createSelectionBounds(x1, y1, x2, y2);
		preselectedElementList.clear();
		for (Iterator i = elementList.iterator(); i.hasNext();) {
			IconElement element = (IconElement) i.next();
			if (selectionBounds.intersects(element.getBounds())) {
				preselectedElementList.add(element);
			}
		}
		Collection inverse = inverseCollection(selectedElementList,
				preselectedElementList);
		preselectedElementList.clear();
		preselectedElementList.addAll(inverse);
		repaint();
	}

	public void decideSelectBoundsAdditional() {
		selectionBounds.setBounds(0, 0, 0, 0);
		selectedElementList.clear();
		selectedElementList.addAll(preselectedElementList);
		preselectedElementList.clear();
		repaint();
	}

	public void readyMoveSelection() {

	}

	public void moveSelection(int dx, int dy) {
		for (Iterator i = selectedElementList.iterator(); i.hasNext();) {
			IconElement element = (IconElement) i.next();
			element.translate(dx, dy);
		}
		repaint();
	}

	public void decideMoveSelection() {
	}

	public void readyCopySelection() {
		List copyElementList = new ArrayList();
		for (Iterator i = selectedElementList.iterator(); i.hasNext();) {
			IconElement element = (IconElement) i.next();
			copyElementList.add(copyElement(element));
		}
		selectedElementList.clear();
		selectedElementList.addAll(copyElementList);
	}

	public void copySelection(int dx, int dy) {
		moveSelection(dx, dy);
	}

	public void decideCopySelection() {
		System.out.println("Rs[m");
	}

	public void edit(Object element) {
		((IconElement)element).setEditale(true);
		repaint();
	}

	/*
	 * WWgċt]
	 */
	private Set inverseCollection(Collection c, Collection inverser) {
		Set result = new HashSet();

		// ϏW߂
		Collection intersection = new LinkedList(c);
		intersection.retainAll(inverser);

		// aWϏW
		result.addAll(c);
		result.addAll(inverser);
		result.removeAll(intersection);

		return result;
	}

}
