/*
 * Created on 2005/12/18
 *
 * Copyright (c) 2005 Nooto Poject.  All rights reserved.
 */
package note.command;

import java.awt.Point;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.Action;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;

import note.NoteApplication;
import note.control.NoteApplicationMode;
import note.model.Group;
import note.model.Page;
import note.model.PictureGroup;
import note.model.TextGroup;
import note.model.TransferableGroupData;
import note.model.picture.PicturePoint;
import note.viewer.PictureGroupPane;
import note.viewer.TextGroupPane;
import note.viewer.*;

/**
 * Class PasteCommand
 * 
 * Nbv{[h̓e\tR}hB
 * 
 * @author camei
 * @version $Id: PasteCommand.java,v 1.12 2006/02/08 20:02:03 camei Exp $
 */
public class PasteCommand extends AbstractCommand {

	/***************************************************************************
	 * 萔
	 **************************************************************************/
	private static final long serialVersionUID = 1L;

	// f[^`
	/** eLXg` */
	public static final DataFlavor TEXT_FLAVOR = DataFlavor.stringFlavor;
	/** teLXg` */
	public static final DataFlavor STYLED_DOCUMENT_FLAVOR = new DataFlavor(
			DefaultStyledDocument.class, "styled document");
	/** }`` */
	public static final DataFlavor PICTURE_FLAVOR = PictureGroup.PICTURE_FLAVOR;

	/** O[v` */
	public static final DataFlavor GROUP_FLAVOR = TransferableGroupData.GROUP_FLAVOR;

	/***************************************************************************
	 * ϐ
	 **************************************************************************/
	// Undo
	private Transferable pastedData;
	private String pastedText = null;// \tꂽeLXgf[^
	private PictureGroup pastedPictureGroup;// \t}`O[vf[^
	private Group[] pastedGroups = null;// \tꂽO[vf[^

	private TextGroup createdTextGroup;// VK쐬eLXgO[v
	private PictureGroup createdPictureGroup;// VK쐬ꂽ}`O[v

	private Page updatedPage = null;// }ꂽy[W
	private PictureGroup updatedPictureGroup;// }\tꂽ}`O[v
	private NoteApplicationMode oldMode = null;
	private PicturePoint oldFocusPoint = null;
	private Point oldAnchorPoint = null;

	private boolean hasPastedNewTextGroup = false;// VKeLXgO[vƂĂ\t
	private boolean hasPastedNewPictureGroup = false;// VK}`O[vƂē\t
	private boolean hasInsertedPicture = false;// }`O[vɐ}`}
	private boolean hasPastedGroup = false;
	private PictureDeleteCommand pictureDeleteCommand = null;// }`폜ꍇ̐}`폜R}h

	/**
	 * RXgN^B
	 * 
	 * @param reciever V[o[
	 */
	public PasteCommand(NoteApplication reciever) {
		super(reciever);
	}

	/**
	 * @return N[߂B
	 */
	public Object clone() {
		return new PasteCommand(getApplication());
	}

	/**
	 * @return ݎs\
	 * @see note.command.AbstractCommand#canExecute()
	 */
	public boolean canExecute() {
		return true;
	}

	/**
	 * @see note.command.AbstractCommand#execute()
	 */
	public void execute() {
		super.execute();

		Toolkit toolKit = Toolkit.getDefaultToolkit();
		Clipboard clp = toolKit.getSystemClipboard();
		pastedData = clp.getContents(this);// Undo̕ۑ

		if (pastedData != null) {// Nbv{[hɃf[^ꍇ
			// \t
			try {
				paste(pastedData);

			} catch (UnsupportedFlavorException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/*
	 * \t
	 */
	private void paste(Transferable data) throws UnsupportedFlavorException,
			IOException {

		// f[^`ɉ\t
		NoteApplicationMode mode = getMode();
		oldMode = (NoteApplicationMode) mode.clone();// Undo̕ۑ
		if (data.isDataFlavorSupported(TEXT_FLAVOR)) {// eLXgf[^̏ꍇ
			pastedText = (String) data.getTransferData(TEXT_FLAVOR);// Undo̕ۑ
			pastePlainTextData(pastedText, mode);

		} else if (data.isDataFlavorSupported(PICTURE_FLAVOR)) {// }`f[^̏ꍇ
			pastedPictureGroup = (PictureGroup) data
					.getTransferData(PICTURE_FLAVOR);// Undo̕ۑ
			pastePictureData(pastedPictureGroup, mode);
		} else if (data.isDataFlavorSupported(GROUP_FLAVOR)) {// O[vf[^̏ꍇ
			pastedGroups = (Group[]) data.getTransferData(GROUP_FLAVOR);// Undo̕ۑ
			pasteGroupData(pastedGroups, mode);
		}
	}

	/*
	 * eLXgf[^\tB
	 */
	private void pastePlainTextData(String text, NoteApplicationMode mode)
			throws UnsupportedFlavorException, IOException {
		if (mode.isTextMode()) {// eLXg̓[h̏ꍇ
			// TextAction܂œ\ts
			// O[vXV̒ʒm Document O[vɒʒm̂ŕKvȂ
			TextGroupPane editor = getEditableTextGroupPane();
			Action cutAction = editor.getActionMap().get("paste");
			ActionEvent e = new ActionEvent(editor,
					ActionEvent.ACTION_PERFORMED, "paste");
			cutAction.actionPerformed(e);

		} else if (mode.isPictureMode()) {// }`̓[h̏ꍇ
			PictureGroupPane editor = getEditablePictureGroupPane();
			if (editor != null && editor.hasSelection()) {// I͈͂ꍇ
				// I͈͂폜
				pictureDeleteCommand = new PictureDeleteCommand(
						PictureDeleteCommand.NOT_CONNECT, getApplication());
				pictureDeleteCommand.execute();
			}

			insertNewTextGroup(text);
			hasPastedNewTextGroup = true;// Undo̕ۑ

			repaint();
		}
	}

	/*
	 * }`f[^\tB
	 */
	private void pastePictureData(PictureGroup pictureData,
			NoteApplicationMode mode) throws UnsupportedFlavorException,
			IOException {

		if (mode.isTextMode()) {// eLXg̓[h̏ꍇ
			// I͈͂폜
			TextGroup textGroup = getEdittingTextGroup();
			if (textGroup.getEditor().hasSelection()) {// I͈͂ꍇ
				textGroup.getTextPane().replaceSelection("");
			}

			// VK}`O[vƂĐ}`}
			insertNewPictureGroup(pictureData);

			// }`O[vm肷
			createdPictureGroup.getEditor().setEditable(false);

			hasPastedNewPictureGroup = true;// Undo̕ۑ

		} else if (mode.isPictureMode()) {// }`̓[h̏ꍇ
			PictureGroup pictureGroup = getEdittingPictureGroup();
			if (mode.isInputting()) {// ͒̏ꍇ

				oldFocusPoint = pictureGroup.getFocusPoint();// Undo̕ۑ

				// I͈͂폜
				PictureGroupPane editor = getEditablePictureGroupPane();
				if (editor.hasSelection()) {// I͈͂ꍇ
					// I͈͂폜
					pictureDeleteCommand = new PictureDeleteCommand(
							PictureDeleteCommand.NOT_CONNECT, getApplication());
					pictureDeleteCommand.execute();
				}

				// ҏW̐}`O[vɐ}`}
				insertPictureGroup(pictureData);// }

				mode.groupUpdated(updatedPictureGroup);

				// pastedPictureGroup = pictureData;// Undo̕ۑ
				hasInsertedPicture = true;// Undo̕ۑ

			} else if (mode.isWaitInputting()) {// ͑ҋ@̏ꍇ

				oldAnchorPoint = (Point) getCurrentPage().getAnchorPoint();// Undo̕ۑ

				// VK}`O[vƂĐ}`}
				updatedPage = getCurrentPage();// Undo̕ۑ
				Point anchorPoint = getCurrentPageViewer().getAnchorPoint();
				createdPictureGroup = new PictureGroup(
						(int) anchorPoint.getX(), (int) anchorPoint.getY());

				// y[Wɐ}`O[vǉ
				updatedPage.addGroup(createdPictureGroup);

				// ҏW\ɂ
				createdPictureGroup.getEditor().setEditable(true);

				// VK}`O[vɐ}`f[^}
				createdPictureGroup.addPictureGroup(pictureData);

				mode.groupUpdated(getEdittingPictureGroup());

				hasPastedNewPictureGroup = true;// Undo̕ۑ
			}

		}
		repaint();
	}

	/*
	 * O[vf[^\tB
	 */
	private void pasteGroupData(Group[] groups, NoteApplicationMode mode)
			throws UnsupportedFlavorException, IOException {
		updatedPage = getCurrentPage();// Undo̕ۑ
		PageViewer pageUI = getCurrentPageViewer();

		// I͈͂
		pageUI.clearSelectedGroups();

		// O[v\t
		for (int i = 0; i < groups.length; i++) {
			updatedPage.addGroup(groups[i]);

			// \tO[vIԂɂ
			pageUI.addSelectedGroup(groups[i]);
		}
		hasPastedGroup = true;// Undo̕ۑ

		repaint();
	}

	/*
	 * ҏW̐}`O[vɐ}`}B
	 */
	private void insertPictureGroup(PictureGroup pictureGroup) {
		updatedPictureGroup = getEdittingPictureGroup();// Undo̕ۑ
		updatedPictureGroup.addPictureGroup(pictureGroup);
	}

	/*
	 * VK}`O[vƂĐ}`}B
	 */
	private void insertNewPictureGroup(PictureGroup pictureGroup) {
		{// }`}}`O[v
			// VK}`O[v쐬
			updatedPage = getCurrentPage();// Undo̕ۑ
			Point anchorPoint = getCurrentPageViewer().getAnchorPoint();
			createdPictureGroup = new PictureGroup((int) anchorPoint.getX(),
					(int) anchorPoint.getY());

			// y[Wɐ}`O[vǉ
			updatedPage.addGroup(createdPictureGroup);

			// VK}`O[vɐ}`f[^}
			createdPictureGroup.addPictureGroup(pictureGroup);
		}

	}

	/*
	 * VKeLXgO[v}B
	 */
	private void insertNewTextGroup(String text) {
		// eLXg}eLXgO[v
		Point anchor = getCurrentPage().getAnchorPoint();
		createdTextGroup = new TextGroup((int) anchor.getX(), (int) anchor
				.getY());
		try {
			createdTextGroup.insertString(0, text);
		} catch (BadLocationException e) {
			// TODO ꂽ catch ubN
			e.printStackTrace();
		}

		// y[Wɑ}
		getCurrentPage().addGroup(createdTextGroup);

		createdTextGroup.getEditor().setEditable(false);// eLXgO[vm肷

		hasPastedNewTextGroup = true;// Undo̕ۑ
	}

	/***************************************************************************
	 * Undo Redo TODO 
	 **************************************************************************/
	/*
	 * @see note.command.AbstractCommand#isUndoableEdit()
	 */
	public boolean isUndoableEdit() {
		return hasPastedNewTextGroup || hasPastedNewPictureGroup
				|| hasInsertedPicture || hasPastedGroup;// \tꍇ͎\
	}

	/*
	 * @see javax.swing.undo.AbstractUndoableEdit#getPresentationName()
	 */
	public String getPresentationName() {
		String dataText = "";
		if (hasPastedNewPictureGroup || hasInsertedPicture) {
			dataText = "}`";
		} else if (hasPastedNewTextGroup) {
			dataText = "eLXg";
		} else if (hasPastedGroup) {
			dataText = "O[v";
		}
		return dataText + "̓\t";
	}

	/*
	 * @see javax.swing.undo.AbstractUndoableEdit#redo()
	 */
	public void redo() throws CannotRedoException {
		super.redo();

		NoteApplicationMode mode = getMode();
		
		// \tf[^gē\t
		try {
			if (hasPastedNewTextGroup) {
				pastePlainTextData(pastedText, mode);

			} else if (hasPastedNewPictureGroup || hasInsertedPicture) {
				pastePictureData(pastedPictureGroup, mode);

			} else if (hasPastedGroup) {// O[v\tꍇ
				pasteGroupData(pastedGroups, mode);
			}
		} catch (UnsupportedFlavorException e) {
			// TODO ꂽ catch ubN
			e.printStackTrace();
		} catch (IOException e) {
			// TODO ꂽ catch ubN
			e.printStackTrace();
		}
	}

	/*
	 * @see javax.swing.undo.AbstractUndoableEdit#undo()
	 */
	public void undo() throws CannotUndoException {
		super.undo();

		Page currentPage = getCurrentPage();
		if (hasPastedNewTextGroup) {
			currentPage.removeGroup(createdTextGroup);

		} else if (hasPastedNewPictureGroup) {
			// \t}`O[v폜
			currentPage.removeGroup(createdPictureGroup);

			if (oldAnchorPoint != null) {// }`̓[hœ\tꍇ
				// AJ[\tÖʒuɈړ
				currentPage.setAnchorLocation(oldAnchorPoint);
			}

		} else if (hasInsertedPicture) {
			// }\t}`B폜
			updatedPictureGroup.removeAll(pastedPictureGroup.getPictureList());

			// \tۂɍ폜}`B𕜊
			if (pictureDeleteCommand != null) {
				pictureDeleteCommand.undo();
			}

			// I_\tOɖ߂
			updatedPictureGroup.setFocusPoint(oldFocusPoint);

		} else if (hasPastedGroup) {// O[v\tꍇ

			// \tO[v폜
			for (int i = 0; i < pastedGroups.length; i++) {
				currentPage.removeGroup(pastedGroups[i]);
			}
		}

		// JڑȌԂɖ߂
		getMode().setValue(oldMode);

		repaint();// ĕ`
	}
}
