/**
 * 
 */
package note.configuration;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import note.configuration.element.AbstractFunctionConfigurationElement;
import note.configuration.element.KeyAndFunctionConfigurationElement;
import note.configuration.element.KeyConfigurationElement;
import note.configuration.element.MoveKeyModifiersConfigurationElement;

import org.apache.xml.serialize.DOMSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * _x̃L[ݒXMLt@Cɏo͂@\B
 * 
 * @author camei
 * @version $Id: KeyConfigurationXMLSerializer.java,v 1.3 2005/11/29 14:13:57
 * camei Exp $
 */
public class KeyConfigurationXMLSerializer {

	/***************************************************************************
	 * 萔
	 **************************************************************************/
	// o̓t@C
	private static final String ANCHOR_MOVE_FILE_NAME = KeyConfigurationXMLParser.ANCHOR_MOVE_KEY_CONFIGURATION_PATH;
	private static final String PICTURE_INPUT_FILE_NAME = KeyConfigurationXMLParser.PICTURE_INPUT_KEY_CONFIGURATION_PATH;
	private static final String FUNCTION_FILE_NAME = KeyConfigurationXMLParser.FUNCTION_KEY_CONFIGURATION_PATH;
	private static final String MISC_FILE_NAME = KeyConfigurationXMLParser.MISC_KEY_CONFIGURATION_PATH;

	// [g^O
	private static final String ANCHOR_MOVE_ROOT_TAG = KeyConfigurationXMLParser.ANCHOR_MOVE_KEY_ROOT_TAG;
	private static final String PICTURE_INPUT_KEY_ROOT_TAG = KeyConfigurationXMLParser.PICTURE_INPUT_KEY_ROOT_TAG;
	private static final String FUNCTION_KEY_ROOT_TAG = KeyConfigurationXMLParser.FUNCTION_KEY_ROOT_TAG;
	private static final String MISC_KEY_ROOT_TAG = KeyConfigurationXMLParser.MISC_KEY_ROOT_TAG;

	private static final String XML_ENCODING = "Shift_JIS";

	/***************************************************************************
	 * 
	 **************************************************************************/

	// XMLDocument
	private Document anchorMoveDocument;// AJ[ړL[XMLDocument
	private Document functionDocument;// @\L[XMLDocument
	private Document pictureDocument;// }`L[XMLDocument
	private Document miscDocument;// ̑L[XMLDocument

	// ^Ovf
	private Element anchorMoveElement;// AJ[ړL[^Ovf
	private Element functionElement;// @\L[^Ovf
	private Element pictureElement;// }`L[^Ovf
	private Element miscElement;// ̑L[^Ovf

	/***************************************************************************
	 * \bh
	 **************************************************************************/

	/**
	 * RXgN^
	 */
	public KeyConfigurationXMLSerializer() {

		// XMLDocument
		try {
			// DocumentBuilder𐶐
			DocumentBuilder documentBuilder = DocumentBuilderFactory
					.newInstance().newDocumentBuilder();

			// eXMLDocument
			anchorMoveDocument = documentBuilder.newDocument();
			functionDocument = documentBuilder.newDocument();
			pictureDocument = documentBuilder.newDocument();
			miscDocument = documentBuilder.newDocument();

		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw new RuntimeException("XMLDocumentł܂");
		}

	}

	/**
	 * _x̃L[ݒXMLt@Cɏo͂B
	 */
	public void serialize(ConceptualKeyConfiguration conceptualKeyConfiguration) {

		{// _x̃L[ݒǂݍ݁Ae[g^Ovf
			// L[Ƌ@\̑Ήݒ肷
			List keyConfigurationList;// eL[ݒ

			keyConfigurationList = conceptualKeyConfiguration
					.getMoveKeyConfigurationList();
			anchorMoveElement = getElementByKeyMapConfiguration(
					keyConfigurationList, ANCHOR_MOVE_ROOT_TAG,
					anchorMoveDocument);
			keyConfigurationList = conceptualKeyConfiguration
					.getPictureKeyConfigurationList();
			pictureElement = getElementByKeyMapConfiguration(
					keyConfigurationList, PICTURE_INPUT_KEY_ROOT_TAG,
					pictureDocument);
			keyConfigurationList = conceptualKeyConfiguration
					.getFunctionKeyConfigurationList();
			functionElement = getElementByKeyMapConfiguration(
					keyConfigurationList, FUNCTION_KEY_ROOT_TAG,
					functionDocument);
			keyConfigurationList = conceptualKeyConfiguration
					.getPictureEdittingKeyConfigurationList();
			miscElement = getElementByKeyMapConfiguration(keyConfigurationList,
					MISC_KEY_ROOT_TAG, miscDocument);

			{// ړCqL[̐ݒ
				// ړCqL[ݒvf
				String modifiersText = conceptualKeyConfiguration
						.getMoveKeyModifiersText();
				MoveKeyModifiersConfigurationElement modifiersConfigurationElement = new MoveKeyModifiersConfigurationElement();
				modifiersConfigurationElement.setKeyText(modifiersText);

				// ړCqL[ݒvfAړCqL[^Ovf
				Element modifiersElement = modifiersConfigurationElement
						.parseXMLElement(anchorMoveDocument);

				// AJ[ړ^OvfɈړCqL[^Ovfǉ
				anchorMoveElement.appendChild(modifiersElement);
			}

		}

		// eXMLDocumentɊe[g^Ovfǉ
		anchorMoveDocument.appendChild(anchorMoveElement);
		pictureDocument.appendChild(pictureElement);
		functionDocument.appendChild(functionElement);
		miscDocument.appendChild(miscElement);

		// eXMLDocumento͂
		outputToXML(ANCHOR_MOVE_FILE_NAME, anchorMoveDocument);
		outputToXML(PICTURE_INPUT_FILE_NAME, pictureDocument);
		outputToXML(FUNCTION_FILE_NAME, functionDocument);
		outputToXML(MISC_FILE_NAME, miscDocument);
	}

	/*
	 * XMLDocumentXMLt@CƂďo͂
	 */
	private void outputToXML(String outputFileName, Document document) {
		try {

			// XMLDocumentXMLt@Cɏo͂
			OutputFormat format = new OutputFormat(document, XML_ENCODING, true);
			DOMSerializer serializer;
			FileOutputStream output = new FileOutputStream(outputFileName);
			serializer = new XMLSerializer(output, format).asDOMSerializer();
			serializer.serialize(document);

		} catch (FileNotFoundException e) {
			e.printStackTrace();
			throw new RuntimeException("ۑpL[ݒXML܂");

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/*
	 * L[ݒ胊Xg烋[g^OvfB
	 */
	private Element getElementByKeyMapConfiguration(List configurationList,
			String tagName, Document document) {

		Element rootElement = document.createElement(tagName);// [g^Ovf

		// L[ݒvfƋ@\ݒvf烋[g^Ovf
		KeyAndFunctionConfigurationElement keyAndFunction;// L[ݒvfƋ@\ݒvf̃Zbg
		Element keyElement;// L[^Ovf
		for (int i = 0; i < configurationList.size(); i++) {

			// L[^Ovf
			keyAndFunction = (KeyAndFunctionConfigurationElement) configurationList
					.get(i);

			keyElement = getKeyElementByKeyAndFunction(keyAndFunction, document);

			// [g^OvfɃL[^Ovfǉ
			rootElement.appendChild(keyElement);
		}

		return rootElement;
	}

	/*
	 * L[ݒvfƋ@\ݒvf̃ZbgAL[^OvfB
	 */
	private Element getKeyElementByKeyAndFunction(
			KeyAndFunctionConfigurationElement keyAndFunction, Document document) {

		KeyConfigurationElement keyConfigurationElement;// L[ݒvf
		AbstractFunctionConfigurationElement functionConfigurationElement;// @\ݒvf
		Element keyElement;// L[^Ovf
		Element functionElement;// @\^Ovf

		// L[^Ovf
		keyConfigurationElement = keyAndFunction.getKey();
		keyElement = keyConfigurationElement.parseXMLElement(document);

		// @\^Ovf
		functionConfigurationElement = keyAndFunction.getFunction();
		functionElement = functionConfigurationElement
				.parseXMLElement(document);

		// L[^Ovfɋ@\^Ovfǉ
		keyElement.appendChild(functionElement);

		return keyElement;
	}

}
