package note.viewer;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.datatransfer.DataFlavor;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;

import note.model.Group;
import note.model.NoteElement;
import note.model.Picture;
import note.model.PictureGroup;
import note.model.picture.PicturePoint;

/**
 * }`O[vGUIGfB^B
 * 
 * @author camei
 * @version $Id: PictureGroupPane.java,v 1.4 2006/02/08 20:02:02 camei Exp $
 */
public class PictureGroupPane implements GroupEditor {

	/***************************************************************************
	 * 萔
	 **************************************************************************/
	// I_̈ړ萔
	/** ړ萔  */
	public static final int UP = 1;
	/** ړ萔  */
	public static final int DOWN = 2;
	/** ړ萔  */
	public static final int LEFT = 3;
	/** ړ萔 E */
	public static final int RIGHT = 4;

	// DataFlavor
	public static final DataFlavor PICTURE_FLAVOR = new DataFlavor(
			PictureGroup.class, "picture");

	private static final int EDITTING_PICTURE_POINT_RADIUS = 2;// }`̕ҏW_̔a
	private static final Stroke PICTURE_STROKE = new BasicStroke(1f);// }`̐

	// nCCg
	private static final BasicStroke HIGHLIGHTER_STROKE = new BasicStroke(6.0f);
	private static final Color HIGHLIGHTER_COLOR = new Color(200, 200, 200);

	private static final int INFINITE_DISTANCE = 3000;// [_

	/***************************************************************************
	 * ϐ
	 **************************************************************************/

	private PictureGroup pictureGroup;// \,ҏWO[v
	private boolean isEditable = false;// ҏW\Ԃ
	private PicturePoint focusPoint = new PicturePoint(new Point(0, 0));// I_
	private PageViewer pageUI;// eO[v

	private Set selectedPictureSet = new HashSet();// IĂ}`<IĂ_瓱o>
	private Set selectedPointSet = new HashSet();// IĂ}`_

	// `ݒ
	private Color edittingPointColor = Color.BLUE;// ҏW̐}`O[v̕ҏW_̐F

	/*
	 * RXgN^B
	 */
	private PictureGroupPane() {
	}

	/**
	 * RXgN^B
	 * 
	 * @param pictureGroup
	 * @param pageUI
	 */
	public PictureGroupPane(PictureGroup pictureGroup, PageViewer pageUI) {
		this();
		this.pictureGroup = pictureGroup;
		this.pageUI = pageUI;
		pictureGroup.setEditor(this);
	}

	/***************************************************************************
	 * GroupEditor ̎
	 **************************************************************************/

	public Group getGroup() {
		return this.pictureGroup;
	}

	public void setEditable(boolean isEditable) {
//		if (this.isEditable != isEditable) {
			this.isEditable = isEditable;
			if (!isEditable) {// m肵ꍇ
				// I͈͂
				selectedPictureSet.clear();
			}

			// PageViewer ɕύXʒmB
			pageUI.groupEditableChanged(this);

			// PageViewer ɃtH[JXڂ
			pageUI.requestFocus();
//		}
	}

	public boolean isEditable() {
		return isEditable;
	}

	public Point getAnchorPoint() {
		return new Point((int) focusPoint.getX(), (int) focusPoint.getY());
	}

	public boolean contains(int x, int y) {
		return pictureGroup.getBounds().contains(x, y);
	}

	public boolean intersects(int x, int y, int width, int height) {
		return pictureGroup.intersects(x, y, width, height);
	}

	public boolean hasSelection() {
		return !selectedPictureSet.isEmpty();
	}

	public void clearSelection() {
		selectedPictureSet.clear();
		selectedPointSet.clear();
	}

	public void insertElement(NoteElement element) {
		// TODO ꂽ\bhEX^u

	}

	public void changedUpdate(Group group) {

	}

	public void movedUpdate(Group group) {
		pageUI.groupAnchorPointMoved(this);
	}

	/***************************************************************************
	 * }`I֘A
	 **************************************************************************/
	/**
	 * I_ړB<br>
	 * ړ͍WIɈړɂAł߂_B<br>
	 * ɓĂ͂܂_Ȃꍇ͈ړȂB<br>
	 */
	public void moveFocusPoint(int direction) {
		// ł𖞂Ă_Ɉړiړ悪Ȃꍇ͈ړȂj
		PicturePoint target = findMovePoint(direction);
		if (target != null) {
			// ړ
			this.focusPoint = target;

			// I͈͂
			clearSelection();

			fireFocusMoved();// I_ړʒm
		}
	}

	/**
	 * I͈͂w肵ȂI_ړB
	 */
	public void moveFocusPointWithSelecting(int direction) {
		PicturePoint target = findMovePoint(direction);
		if (target != null) {
			// ݂̓_IĂ_ɂ
			selectedPointSet.add(focusPoint);

			// ړ
			this.focusPoint = target;

			// ړ̓_IĂ_ɂ
			selectedPointSet.add(focusPoint);

			// I͈͂XV
			updateSelection();

			fireFocusMoved();// I_ړʒm
		}
	}

	/*
	 * ړ̓_TB ړ̓_Ȃꍇ<code>null<code>߂B
	 */
	private PicturePoint findMovePoint(int direction) {
		List picturePointList = pictureGroup.getPicturePointList();

		PicturePoint movePoint = null; // ړ̓_
		double minDistance = INFINITE_DISTANCE; // ҏW_Ƃ̍ŒZl
		double x = this.focusPoint.getX(); // ړO̓_x
		double y = this.focusPoint.getY(); // ړO̓_y

		// SĂ̓_ɑ΂ď𖞂Ă邩ׂ TODO ǂ@l
		for (int i = 0; i < picturePointList.size(); i++) {
			PicturePoint point = (PicturePoint) picturePointList.get(i);
			if (point.equals(this.focusPoint)) {
				continue; // ݂̓_͈ړΏۂɂȂȂ
			}
			// ړ̏𖞂Ăꍇ
			if ((direction == UP && point.getY() < y)
					|| (direction == DOWN && point.getY() > y)
					|| (direction == LEFT && point.getX() < x)
					|| (direction == RIGHT && point.getX() > x)) {
				// _Ƃ̋ŒZǂׂ
				double distance = point.distance(this.focusPoint);
				if (distance < minDistance) {
					minDistance = distance; // ŒZXV
					movePoint = point;
				}
			}
		}

		return movePoint;
	}

	/**
	 * }`IB
	 * 
	 * @param picture I}`
	 * @return <code>true</code> }`܂łꍇ
	 */
	public boolean selectPicture(Picture picture) {
		if (pictureGroup.getPictureList().contains(picture)) {
			selectedPictureSet.add(picture);
			return true;
		} else {
			return false;
		}
	}

	/**
	 * }`BIB
	 * 
	 * @param pictures I}`B
	 * @return <code>true</code> }`܂łꍇ
	 */
	public boolean selectPictures(Collection pictures) {
		if (pictureGroup.getPictureList().containsAll(pictures)) {
			selectedPictureSet.addAll(pictures);
			return true;
		} else {
			return false;
		}
	}

	/**
	 * }`I珜OB
	 * 
	 * @param picture I珜O}`
	 * @return <code>true</code> }`܂łꍇ
	 */
	public boolean unselectPicture(Picture picture) {
		return selectedPictureSet.remove(picture);
	}

	/**
	 * @return IĂ}`̃Xg
	 */
	public List getSelectedPictureList() {
		return new ArrayList(selectedPictureSet);
	}

	/**
	 * I͈͂߂B<br>
	 * 
	 * @return IĂ}`Bō\}`O[v߂B
	 */
	public Object getSelection() {
		PictureGroup selection;

		// {// I͈͂\}`O[v
		// // I͈͂̎n__Ƃ}`O[v
		// PicturePoint begin = getBegin(selectedPictureSet);
		// int x = (int) begin.getX();
		// int y = (int) begin.getY();
		// selection = new PictureGroup(x, y);
		//
		// // }`O[vɑI͈͂̐}`Bǉ
		// selection.pictureList.addAll(selectedPictureSet);
		//
		// // fB[vRs[쐬
		// selection = (PictureGroup) selection.clone();
		//
		// // I͈͂̎n__ɂȂ悤ɑSĂ̐}`ړ
		// translatePictures(selection.pictureList, -x, -y);
		// }
		//
		// return selection;
		return null;
	}

	/*
	 * I͈͂XVBIĂ_ωƂɌĂяo\bhB
	 */
	private void updateSelection() {
		// IĂ_Qȏ܂ސ}`I
		List pictureList = pictureGroup.getPictureList();
		for (ListIterator i = pictureList.listIterator(); i.hasNext();) {
			int count = 0;
			Picture picture = (Picture) i.next();// }`

			// }`̎ҏW_̒ɁAIĂ_܂܂Ă邩
			for (Iterator j = selectedPointSet.iterator(); j.hasNext();) {
				PicturePoint selected = (PicturePoint) j.next();

				if (picture.contains(selected)) {
					count++;
				}
			}

			// IĂ_Qȏ܂łΑI
			if (count >= 2) {
				selectedPictureSet.add(picture);
			}
		}
	}

	/***************************************************************************
	 * }`ό`֘A
	 **************************************************************************/
	/**
	 * I_ړĐ}`ό`B<br>
	 * W̓y[WWnŎw肷邱ƁB<br>
	 * 
	 * @param x ړx
	 * @param y ړy
	 */
	public void edittingMoveByFocusPoint(int x, int y) {
		edittingMove(focusPoint, x, y);
	}

	/**
	 * }`_ړĐ}`ό`B<br>
	 * W̓y[WWnŎw肷邱ƁB<br>
	 * 
	 * @param x ړx
	 * @param y ړy
	 */
	public void edittingMove(PicturePoint picturePoint, int x, int y) {
		// O[vWnɕϊ
		int localX = x - pictureGroup.getX();
		int localY = y - pictureGroup.getY();

		// I_ҏWړ
		pictureGroup.edittingMoveFocusPoint(picturePoint, localX, localY);

		fireFocusMoved();// I_Ƃʒm
		fireGroupChanged();// O[vύXꂽƂʒm
	}

	/***************************************************************************
	 * }`폜֘A
	 **************************************************************************/
	/**
	 * I_̒̐}`폜B<br>
	 * 
	 * @return 폜}`B
	 */
	public Collection removeFocusNext() {
		Collection children = focusPoint.getChildPictures();// I_̎q}`
		List removedPictures = new ArrayList();// 폜}`B

		if (children.isEmpty()) {// I_n_Ƃ}`ꍇ
			Picture parent = focusPoint.getParentPicture();

			if (parent != null) {
				// I_qƂ}`폜
				pictureGroup.remove(parent);
				removedPictures.add(parent);// 폜}`L^

				// I_폜}`̎n_ɐݒ肷
				focusPoint = parent.getBeginPoint();
			}

		} else {// I_n_Ƃ}`ꍇ
			// I_n_Ƃ}`폜
			pictureGroup.removeAll(children);
			removedPictures.addAll(children);// 폜}`L^
		}

		// I_폜ꂽꍇA߂_Ɉړ
		pictureGroup.getPicturePointList().contains(focusPoint);
		PicturePoint nearPoint = findNearPoint(focusPoint);
		if (nearPoint != null) {
			focusPoint = nearPoint;
		}

		fireFocusMoved();// I_Ƃʒm
		fireGroupChanged();// O[vύXꂽƂʒm
		return removedPictures;
	}

	/**
	 * I_̒O̐}`폜B<br>
	 * O̐}`ꍇ͍폜ȂB
	 * 
	 * @return 폜}`
	 */
	public Picture removeFocusPrevious() {
		// 폜Ώۂ߂
		Picture removeTarget = focusPoint.getParentPicture();// 폜Ώۂ̐}`

		// 폜
		if (removeTarget != null) {// 폜Ώۂ̐}`ꍇ
			// I_폜}`̎n_Ɉړ
			focusPoint = removeTarget.getBeginPoint();

			// 폜
			pictureGroup.remove(removeTarget);
		}

		// I_폜ꂽꍇA߂_Ɉړ
		pictureGroup.getPicturePointList().contains(focusPoint);
		PicturePoint nearPoint = findNearPoint(focusPoint);
		if (nearPoint != null) {
			focusPoint = nearPoint;
		}

		fireFocusMoved();// I_Ƃʒm
		fireGroupChanged();// O[vύXꂽƂʒm
		return removeTarget;
	}

	/**
	 * I͈͂̐}`폜B
	 * 
	 * @return 폜}`B
	 */
	public Collection removeSelection() {
		Collection removedPictures = new ArrayList();
		if (!selectedPictureSet.isEmpty()) {// I͈͂ꍇ
			removedPictures.addAll(selectedPictureSet);// 폜}`BLĂ

			// IĂ}`폜
			pictureGroup.removeAll(selectedPictureSet);

			// I͈͂
			clearSelection();

			// I_폜ꂽꍇ́A߂_Ɉړ
			if (!pictureGroup.getPicturePointList().contains(focusPoint)) {// I_폜ꂽꍇ
				PicturePoint target = findNearPoint(focusPoint);// ߂_߂
				if (target != null) {// ړ_ꍇ
					focusPoint = target;
				}
			}
		}

		fireFocusMoved();// I_Ƃʒm
		fireGroupChanged();// O[vύXꂽƂʒm
		return removedPictures;
	}

	/*
	 * I_ړƂ PageViewer ɒʒmB
	 */
	private void fireFocusMoved() {
		pageUI.groupAnchorPointMoved(this);
	}

	/*
	 * O[vύXꂽƂ PageViewer ɒʒmB
	 */
	private void fireGroupChanged() {
		pageUI.groupChanged(this);
	}

	/*
	 * ߂_TB ߂_ꍇnull߂B
	 */
	private PicturePoint findNearPoint(Point2D point) {
		PicturePoint foundPoint = null;
		double minDistance = INFINITE_DISTANCE;

		// ł߂_T
		for (Iterator i = pictureGroup.getPicturePointList().iterator(); i
				.hasNext();) {
			PicturePoint target = (PicturePoint) i.next();
			double distance = point.distance(target);
			if (distance < minDistance) {
				minDistance = distance;
				foundPoint = target;
			}
		}
		return foundPoint;
	}

	/***************************************************************************
	 * `֘A
	 **************************************************************************/
	/**
	 * `揈B
	 * 
	 * @param g Graphics
	 */
	public void paint(Graphics2D g2) {

		// }`̃nCCg`悷
		if (hasSelection()) {// I͈͂ꍇ
			g2.setStroke(HIGHLIGHTER_STROKE);
			List selectedPictureList = pictureGroup.getSelectedPictureList();
			for (int i = 0; i < selectedPictureList.size(); i++) {
				// }`1̃nCCg`悷
				Picture picture = (Picture) selectedPictureList.get(i);
				Color color = HIGHLIGHTER_COLOR;// nCCgF
				g2.setPaint(color);
				g2.draw(picture.getShape());
			}
		}

		// }``悷
		g2.setStroke(PICTURE_STROKE);
		List pictureList = pictureGroup.getPictureList();
		for (int i = 0; i < pictureList.size(); i++) {
			// }`P`悷
			Picture picture = (Picture) pictureList.get(i);
			g2.setPaint(picture.getColor());
			g2.draw(picture.getShape());
		}

		// ҏWȂҏW_`悷
		PicturePoint point;
		if (isEditable()) {
			// O[v̑SĂ̕ҏW_`
			List picturePointList = pictureGroup.getPicturePointList();
			g2.setColor(edittingPointColor);
			for (int i = 0; i < picturePointList.size(); i++) {
				point = (PicturePoint) picturePointList.get(i);
				int x = (int) point.getX() - EDITTING_PICTURE_POINT_RADIUS;
				int y = (int) point.getY() - EDITTING_PICTURE_POINT_RADIUS;
				g2.fillOval(x, y, EDITTING_PICTURE_POINT_RADIUS * 2,
						EDITTING_PICTURE_POINT_RADIUS * 2); // _`
			}
		}
	}

	/***************************************************************************
	 * Getter & Setter
	 **************************************************************************/
	/**
	 * @return I_
	 */
	public PicturePoint getFocusPoint() {
		return this.focusPoint;
	}

	/**
	 * I_ݒ肷B
	 * 
	 * @param focusPoint }`O[vɊ܂܂Ă}`_
	 */
	public void setFocusPoint(PicturePoint focusPoint) {
		if (pictureGroup.containsPicturePoint(focusPoint)) {// }`O[vɎw肳ꂽ}`_܂܂Ăꍇ
			this.focusPoint = focusPoint;
			fireFocusMoved();// I_Ƃʒm
		}
	}

}
